#include <QFile>
#include <QDomDocument>
#include <QDebug>
#include <QMessageBox>
#include <QAction>

#include "DataBase.h"

DataBase::DataBase(QObject *parent) :
    QObject(parent)
{
}


//----------------
void DataBase::loadData(QString fileName){

    if (fileName.isEmpty()){
        qDebug()<<"fileName is not a valid fileName";
        exit(1);
    }

    QFile file(fileName);
    if (!file.open(QFile::ReadOnly | QFile::Text)) {
        QMessageBox::warning(0, tr("DOM Bookmarks"),
                             tr("Cannot read file %1:\n%2.")
                             .arg(fileName)
                             .arg(file.errorString()));
        exit(1);
    }

    QString errorStr;
    int errorLine;
    int errorColumn;

    QDomDocument domDocument;

    if (!domDocument.setContent(&file, true, &errorStr, &errorLine,
                                &errorColumn)) {
        QMessageBox::information(0, tr("DOM Menus"),
                                 tr("Parse error at line %1, column %2:\n%3")
                                 .arg(errorLine)
                                 .arg(errorColumn)
                                 .arg(errorStr));
        qDebug()<<"load xml file: error line: "<<errorLine << " error column: "<<errorColumn<<" type: "<<errorStr;
    }

    m_data = domDocument;

    process();

}


void DataBase::process(){
    QDomElement root =  m_data.firstChildElement("root");
    if(root.isNull()){
        qDebug()<<"there is no ROOT element";
        exit(1);
    }

    //qDebug()<<"root OK";
    QDomElement domMenubar = root.firstChildElement("menubar");
    int applicationsCount =0;
    while(!domMenubar.isNull()){
        applicationsCount++;
        processMenuBar(domMenubar);
        domMenubar = domMenubar.nextSiblingElement("menubar");
    }
    qDebug()<< m_menuBar;
}

QString nameItem(QDomElement item){
    QString name = item.attribute("title");
    if(name.isEmpty())
        return "";
    if(name.at(name.size()-1).unicode()==8230){
        name = name.mid(0,name.size()-1);
    }else if(name.endsWith("...")){
        name = name.mid(0, name.size()-3);
    }

    return name;
}

void DataBase::processMenuBar(QDomElement n){
    QString appName = n.attribute("appname");
    QMenuBar* menuBar = new QMenuBar(0);
    menuBar->setObjectName(appName);
    menuBar->setNativeMenuBar(false);
    m_menuBar.push_back(menuBar);
    m_domMenuBar.push_back(n);

    QDomElement domMenu = n.firstChildElement("menu");
    while(!domMenu.isNull()){
        processMenu(domMenu,0, menuBar);
        domMenu = domMenu.nextSiblingElement("menu");
    }

}

QMenuBar* DataBase::menuBar(QDomElement n){
    QString appName = n.attribute("appname");
    QMenuBar* bar = new QMenuBar(0);
    bar->setObjectName(appName);
    bar->setNativeMenuBar(false);

    QDomElement domMenu = n.firstChildElement("menu");
    while(!domMenu.isNull()){
        qDebug()<<"attached Menu";
        processMenu(domMenu,0, bar);
        domMenu = domMenu.nextSiblingElement("menu");
    }
    return bar;
}

void DataBase::processMenu(QDomElement domMenu, QMenu* parentMenu, QMenuBar* bar){
    QString title = domMenu.attribute("title");
    QMenu* menu = 0;
    if(parentMenu){
        menu = parentMenu->addMenu(title);
    }else{
        menu = bar->addMenu(title);
    }

    QDomElement domItem = domMenu.firstChildElement();
    while(!domItem.isNull()){
        if(domItem.tagName()=="menuitem"){
            QAction* action  = menu->addAction( domItem.attribute("title") );


            bool b = domItem.hasAttribute("hotkeyChar");
            if(b){
                QString key = domItem.attribute("hotkeyChar");
                QString modifiers = domItem.attribute("modifiers");
                QString m = "";
                if(modifiers=="0"){
                    m = "Ctrl";
                }else if(modifiers=="1"){
                    m = "Ctrl+Shift";
                }else if(modifiers=="2"){
                    m = "Ctrl+Alt";
                }else if(modifiers=="3"){
                    m = "Ctrl+Shift+Alt";
                }else if(modifiers=="4"){
                    m = "Ctrl+Shift";
                }else if(modifiers=="5"){
                    m = "Ctrl+Shift";
                }
                QKeySequence seq(m+"+"+key);
                action->setShortcut(seq);
            }





        }else if(domItem.tagName()=="separator"){
            menu->addSeparator();

        }else if(domItem.tagName()=="menu"){
            if(nameItem(domItem)!= "Open Recent"){
                processMenu(domItem, menu, bar);
            }else{
               menu->addMenu("Open Recent...");
            }
        }
        domItem = domItem.nextSiblingElement();
    }
}


