#include <QVBoxLayout>
#include <QRadioButton>
#include <QSignalMapper>
#include <QScrollArea>
#include <QDebug>
#include <QPixmap>
#include "MenuVisu.h"

#include "MainWindow.h"

//------------------
MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent)
{
    m_dockWidget = new QDockWidget(tr("Menus"), this);
    m_dockWidget->setAllowedAreas( Qt::LeftDockWidgetArea );
    //m_dockWidget->setMinimumWidth(150);
    addDockWidget(Qt::LeftDockWidgetArea, m_dockWidget);

    QDockWidget* controlDock = new QDockWidget("Control", this);
    controlDock->setAllowedAreas( Qt::RightDockWidgetArea );
    addDockWidget(Qt::RightDockWidgetArea, controlDock);
    QWidget* w = new QWidget(0);
    w->setMinimumSize(100,30);
    controlDock->setWidget(w);

    QPushButton* zoomPlus = new QPushButton("+", w);
    connect(zoomPlus, SIGNAL(clicked()), this, SLOT(increaseZoom()));
    zoomPlus->move(0,0);
    zoomPlus->show();

    QPushButton* zoomMinus = new QPushButton("-", w);
    connect(zoomMinus, SIGNAL(clicked()), this, SLOT(decreaseZoom()));
    zoomMinus->show();
    zoomMinus->move(50,0);



    m_scrollArea = new QScrollArea(0);
    setCentralWidget(m_scrollArea);

    connect(this, SIGNAL(clicked(int)), this, SLOT(activateMenuBar(int)));
    move(30,30);
    resize(1200,700);
    m_zoomLevel = 1.0;

}

//-------------------
void MainWindow::loadMenuBars(QVector<QMenuBar *>menus){
    m_menuBars = menus;
    QWidget *menuChooser = new QWidget();
    QVBoxLayout *layout = new QVBoxLayout(menuChooser);
    menuChooser->setLayout(layout);


    QSignalMapper *signalMapper = new QSignalMapper(this);
    connect(signalMapper, SIGNAL(mapped(int)),
            this, SIGNAL(clicked(const int)));

    for(int i=0;i<menus.size();i++){
        QRadioButton *button = new QRadioButton(menus[i]->objectName(), menuChooser);
        layout->addWidget(button);
        connect(button, SIGNAL(clicked()), signalMapper, SLOT(map()));
        signalMapper->setMapping(button,i);
    }

    QScrollArea *scrollArea = new QScrollArea(m_dockWidget);
    scrollArea->setMinimumWidth(250);
    m_dockWidget->setWidget(scrollArea);
    scrollArea->setWidget(menuChooser);
}

//-------------------
void MainWindow::loadDomMenuBars(){
    m_domMenuBars = m_dataBase->getDomMenuBars();
    QWidget *menuChooser = new QWidget();
    QVBoxLayout *layout = new QVBoxLayout(menuChooser);
    menuChooser->setLayout(layout);


    QSignalMapper *signalMapper = new QSignalMapper(this);
    connect(signalMapper, SIGNAL(mapped(int)),
            this, SIGNAL(clicked(const int)));

    for(int i=0;i<m_domMenuBars.size();i++){
        QString name = m_domMenuBars[i].attribute("appname");
        QRadioButton *button = new QRadioButton(name, menuChooser);
        if(i==0) button->setChecked(true);
        layout->addWidget(button);
        connect(button, SIGNAL(clicked()), signalMapper, SLOT(map()));
        signalMapper->setMapping(button,i);
    }

    QScrollArea *scrollArea = new QScrollArea(m_dockWidget);
    scrollArea->setMinimumWidth(250);
    m_dockWidget->setWidget(scrollArea);
    scrollArea->setWidget(menuChooser);


    activateMenuBar(0);
}

//-------------------------
void MainWindow::increaseZoom(){
    this->m_zoomLevel *=2.;
    zoom();
}

//-------------------------
void MainWindow::decreaseZoom(){
    this->m_zoomLevel /=2.;
    zoom();
}


//-----------------------
void MainWindow::zoom(){
    QPixmap pixmap = pix.scaled(pix.width()*m_zoomLevel, pix.height()*m_zoomLevel);
    QPalette palette;
    palette.setBrush(this->backgroundRole(), QBrush(pixmap));
    sheet->resize(pixmap.size());
    sheet->setPalette(palette);
}

//----------------------
void MainWindow::activateMenuBar(int i){
    QMenuBar* bar = this->m_dataBase->menuBar(m_domMenuBars[i]);
    qDebug()<<"activate menuBar "<< bar->objectName();
    setMenuBar(bar);
    menuBar()->show();

    MenuVisu* unfoldedView = new MenuVisu(bar);
    pix = QPixmap::grabWidget(unfoldedView);
    //     pixmap.save(path+ directory+"menus/"+imgFileName);
    sheet = new QWidget();
    m_scrollArea->setWidget(sheet);
    zoom();
}

//----------------------
