#include <QDebug>
#include <QPainter>
#include <QPen>
#include <QToolButton>
#include <QPolygon>

#include "MenuVisu.h"
#include "MenuM.h"

MenuVisu::MenuVisu(QMenuBar *menuBar, QWidget *parent) :
    QWidget(parent)
{
    m_menuBar = menuBar;

    for(int i=0;i<11;i++){
        m_cumulSubmenu.push_back(0);
    }

    QList<QObject*> widgets = m_menuBar->children();

    for(int i=1; i<widgets.size();i++){
        QMenu* menu = qobject_cast<QMenu*>(widgets[i]);
        MenuM* menuM = new MenuM(menu);
        qDebug()<< "menu "<< menuM->title();
        m_menus.push_back(menuM);
    }
    parseMenus(m_menus);
    //qDebug()<< this->m_posImg;
}


//-------------------------
void MenuVisu::paintEvent(QPaintEvent *){
    QPainter p(this);
    p.setRenderHint(QPainter::Antialiasing);
    QPen pen(Qt::black);
    pen.setWidth(0);
    //pen.setWidth(1);
    //p.setPen(pen);
    QColor c(255,255,255,255);
    p.fillRect(0,0,width(), height(), Qt::black);
    pen.setColor(Qt::white);
    p.setPen(pen);
    p.drawRect(0,0,width(), height());

    QMapIterator<QMenu*, QVector<QPoint> > j(m_linkSubmenu);
    while(j.hasNext()){
        j.next();
        QVector<QPoint> link = j.value();
        QPolygon poly(link);
        p.save();
        p.setPen(Qt::blue);
        QColor c= Qt::blue;
        c.setAlpha(100);
        p.setBrush(c);
        p.translate(10,10);
        p.drawPolygon(poly);
        p.restore();
    }


    QMapIterator<QMenu*,QPixmap> i(m_screenShot);
    while(i.hasNext()){
        i.next();
        pen.setColor(c);
        p.setPen(pen);

        QMenu* m = i.key();
        QPoint pos = this->m_posImg[m];
        p.drawPixmap(10+pos.x(), 10+pos.y(), i.value());
        QRect rect;
        rect.setX(10+pos.x());
        rect.setY(10+pos.y());
        rect.setWidth( i.value().width());
        rect.setHeight(i.value().height());
        pen.setColor(Qt::gray);
        p.setPen(pen);
 //       rect.setTopLeft(QPoint(10+pos.x(),10+pos.y()));
        p.drawRoundRect(rect,3,3);

        if(m->menuAction()->associatedWidgets().size()==0){
            //qDebug()<<"draw title ...";
            QPixmap title = actionPixmap(m->menuAction());
            p.drawPixmap(10+pos.x(), 10+pos.y()-title.height(),title);
        }
    }


}

//---------------------
void MenuVisu::parseMenus(QList<QMenu *>){
    QPoint p(10,20);
       QRect res(0,0,0,0);
       for(int i=0; i<m_menus.size();i++){
           resetCumulSubmenu();
           //qDebug()<<" debug p "<< p;
           QRect rect = parseMenu(m_menus.at(i),p);
           p.setX(rect.x() + rect.width() + 10);
           res.setWidth(p.x());
           if(rect.height()> res.height()){
               res.setHeight(rect.height());
           }
       }

       res.adjust(0,0,20,20);
       resize(res.width(), res.height());
       setMinimumSize(size());
       setMaximumSize(size());
}

//-------------------------
QRect MenuVisu::parseMenu(QMenu* menu, QPoint posParent){
    menu->show();
    QPixmap pixmap = QPixmap::grabWidget(menu);
    //QPixmap pixmap(widget->size());
    //widget->render(&pixmap);
    menu->hide();

    m_screenShot.insert(menu, pixmap);
    m_posImg.insert(menu,posParent);
    m_cumulSubmenu[0] = pixmap.height();

    QList<QAction*> actions = menu->actions();
    for(int i=0; i<actions.size();i++){
        QAction* act = actions.at(i);
        if(act->menu()){
            QRect rectAction = menu->actionGeometry(act);
            QVector<QPoint> link;
            QPoint topLeft(m_posImg[menu].x()+pixmap.width(), m_posImg[menu].y()+rectAction.y());
            QPoint bottomLeft(m_posImg[menu].x()+pixmap.width(), m_posImg[menu].y()+rectAction.y()+rectAction.height());

            link.push_back(bottomLeft);
            link.push_back(topLeft);

            MenuM* subMenuM = new MenuM(act->menu(), menu);
            act->setMenu(subMenuM);
            parseMenu_private(menu, subMenuM, 1, link);
        }
    }

    QRect rect = menuBoundingBox();
    return rect;
}


void MenuVisu::parseMenu_private(QMenu* parentMenu, QMenu* menu, int code, QVector<QPoint> link){
    menu->show();
    QPixmap pixmap = QPixmap::grabWidget(menu);
 //   QPixmap pixmap(menu->size());
 //   menu->render(&pixmap);
    menu->hide();
    m_screenShot.insert(menu, pixmap);

    QRect gact = parentMenu->actionGeometry(menu->menuAction());
    int x = m_posImg[parentMenu].x() + m_screenShot[parentMenu].width() + 20;
    int y = m_posImg[parentMenu].y() + gact.y();
    int d_menu = code;//depthFromMenu(menu);
    if(y< m_cumulSubmenu[d_menu]){
        y = m_cumulSubmenu[d_menu];
    }
    QPoint posMenu = QPoint(x,y);
    this->m_posImg.insert(menu, posMenu);
    updateCumulSubmenu(d_menu, y+pixmap.height());

    QPoint topRight = posMenu;
    QPoint bottomRight(posMenu.x(), posMenu.y()+pixmap.height());
    link.push_back(topRight);
    link.push_back(bottomRight);
    if(link.size()==4){
        this->m_linkSubmenu.insert(menu, link);
    }

    QList<QAction*> actions = menu->actions();
    for(int i=0; i<actions.size();i++){
        QAction* act = actions.at(i);
        if(act->menu()){
            QVector<QPoint> link2;
            parseMenu_private(menu, act->menu(), code+1, link2);
        }
    }
}

//--------------
QRect MenuVisu::menuBoundingBox(){
    int maxX =0;
    int maxY = 0;

    QMapIterator<QMenu*,QPixmap> i(m_screenShot);
    while(i.hasNext()){
        i.next();
        QPixmap pix = i.value();
        QPoint pos = m_posImg[i.key()];
        //qDebug()<<"pos "<<pos.x()<< " width "<<pix.width() <<" maxX "<<maxX;
        if(maxX < pos.x()+pix.width()){
            maxX = pos.x()+pix.width();
        }

        if(maxY < pos.y()+pix.height()){
            maxY = pos.y()+pix.height();
        }
    }
    return QRect(0,0,maxX, maxY);
}

//----------------------
QPixmap MenuVisu::actionPixmap(QAction* action){
    if(!action)
        return QPixmap();
    QToolButton *tb = new QToolButton;
    tb->setText(action->text());
    tb->setToolButtonStyle(Qt::ToolButtonTextOnly);
#ifdef Q_WS_WIN // Force alien off to make adjustSize() take the system minimumsize into account.
    tb->createWinId();
#endif
    tb->adjustSize();
    const QPixmap rc = QPixmap::grabWidget(tb);
    tb->deleteLater();
    return rc;
}


//------------------------
void MenuVisu::updateCumulSubmenu(int d, int y){
    for(int j = d; j >=0; j--){
        if(m_cumulSubmenu[j] < y + 20){
            m_cumulSubmenu[j] = y+20;
        }
    }
}


//-----------------------
void MenuVisu::resetCumulSubmenu(){
    for(int i=0;i<m_cumulSubmenu.size();i++){
        m_cumulSubmenu[i]=0;
    }
}
